#include <switch.h>
typedef struct {
	switch_sockaddr_t *sa;
	switch_socket_t *socket;
	switch_mutex_t *audio_lock;
	switch_buffer_t *audio_buffer;
	char voice_name[50];
	int rate;
	int done;
	int done_gen;
} festival_t;

static char voice_name[50] = "cmu_us_slt_arctic_hts";
static char festival_server[50] = "127.0.0.1";
static switch_port_t festival_port = 1314;
static switch_memory_pool_t *module_pool = NULL;	
static switch_memory_pool_t *memory_pool;
static void festival_printf(switch_socket_t *socket, char *fmt, ...);
static int festival_get_line(char *buf, festival_t *festival);
static char *socket_receive_file_to_buff(festival_t *fd,int *size);
#define RESULT_SEPERATOR "\r\n"

int main(void) 
{
	char *ppath = NULL; 
	const char *err = NULL;
	festival_t *festival;
	int bytes = 0;
	char buf[2048] = "";
	uint8_t crcount = 0;
	uint8_t count = 0;
	char mbuf[1024] = "";
	int filesize;
	FILE *fp;
	char *wavefile;

	if (switch_core_init_and_modload(ppath, &err) != SWITCH_STATUS_SUCCESS) { 
        fprintf(stderr, "Cannot Initilize [%s]\n", err);
        return 255;
    }

	switch_core_new_memory_pool(&memory_pool);
	switch_core_new_memory_pool(&module_pool);

	festival = switch_core_alloc(memory_pool, sizeof(*festival));
	
	if (switch_sockaddr_info_get(&festival->sa, festival_server, AF_INET, festival_port, 0, memory_pool) != SWITCH_STATUS_SUCCESS) {
		return -1;
	}

	if (switch_socket_create(&festival->socket, festival->sa->family, SOCK_STREAM, 0, memory_pool) != SWITCH_STATUS_SUCCESS) { 
		return -1;
	}
	
	if (switch_socket_connect((festival->socket), festival->sa) != SWITCH_STATUS_SUCCESS) {
		return -1;
	}
	printf("FESTIVAL Setting the voice to %s.\n", voice_name);
	festival_printf(festival->socket, "(voice_%s)\n", voice_name);
	while(strncmp(mbuf, "ft_StUfF_keyOK", 14)) {
		festival_get_line(mbuf, festival);
	}
	printf("FESTIVAL Setting the format to raw.\n");
	festival_printf(festival->socket, "(Param.set 'Wavefiletype 'raw)\n");
	while(strncmp(mbuf, "ft_StUfF_keyOK", 14)) {
		festival_get_line(mbuf, festival);
	}	
	printf("FESTIVAL Sending text to Festival.\n");
	festival_printf(festival->socket, "(utt.send.wave.client (utt.wave.resample (utt.wave.rescale (utt.synth" \
					"(eval (list 'Utterance 'Text \"Test\"))) 5) 8000))");
	printf("FESTIVAL we have raw data to read.\n");
	while(strncmp(mbuf, "WV", 2)) {
		festival_get_line(mbuf, festival);
	}	
	printf("FESTIVAL End of raw data.\n");
	while(strncmp(mbuf, "ft_StUfF_key", 12)) {
		festival_get_line(mbuf, festival);
	}	
	printf("FESTIVAL Synthesis Complete.\n");
	festival_printf(festival->socket, "(quit)");
	while(strncmp(mbuf, "ER", 2)) {
		festival_get_line(mbuf, festival);
	}	
	switch_socket_close(festival->socket);

	return switch_core_destroy();
}

static int festival_get_line(char *buf, festival_t *festival)
{
	switch_status_t status = SWITCH_STATUS_SUCCESS; 
	char *ptr; 
	switch_size_t mlen;

	memset(buf, 0, strlen(buf));
	
	ptr = buf;
	while(festival->socket) {
		mlen = 1;
		status = switch_socket_recvfrom(festival->sa, festival->socket, 0, ptr, &mlen);		
		if(*ptr == '\n') {
			ptr = '\0';
			*ptr++;
			break;
		}
		*ptr++;
	}
	return 1;
}

static void festival_printf(switch_socket_t *socket, char *fmt, ...)
{
	char *stuff;
	size_t res = 0, len = 0;

	va_list ap;
	va_start(ap, fmt);
#ifndef vasprintf
	stuff = (char *) malloc(10240);
	vsnprintf(stuff, 10240, fmt, ap);
#else
	res = vasprintf(&stuff, fmt, ap);
#endif
	va_end(ap);
	if (res == -1) {
		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Out of memory\n");
	} else {
		len = strlen(stuff);
		switch_socket_send(socket, stuff, &len);
		
		free(stuff);
	}


}
